// smart_assert_context.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_CONTEXT_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_CONTEXT_HPP_INCLUDED

#include <string>
#include <vector>

namespace boost { 

namespace smart_assert {


/*
    contains the context in which an ASSERT appeared
*/
struct assert_context {

    assert_context() {
        m_nLevel = lvl_debug;
    }
    ////////////////////////////////////////////
    // context

    typedef std::map< std::string, std::string> context_vals_collection;
    const context_vals_collection & get_context_vals_collection() const {
        return m_collContextVals;
    }

    // helper
    std::string get_context_val( const std::string & str) const {
        context_vals_collection::const_iterator
            found = m_collContextVals.find( str);
        if ( found != m_collContextVals.end())
            return found->second;
        else
            return "";
    }

    template< class type>
    void add_context_val( const char * str, const type & val) {
        std::stringstream out;
        out << val;
        m_collContextVals[ str] = out.str();
    }


    ////////////////////////////////////////////
    // expression
    void set_expr( const std::string & str) {
        m_strExpr = str;
    }

    const std::string & get_expr() const {
        return m_strExpr;
    }


    ////////////////////////////////////////////
    // vals

    struct val_and_str {
        val_and_str( const std::string & val, const std::string & str)
            : m_val( val), m_str( str) {}

        // the value, converted to string
        //
        // Example:
        // int *p = 0; SMART_ASSERT(p);
        // m_val = '00000000'
        std::string m_val;
        // the string corresponding to the value
        //
        // Example:
        // int *p = 0; SMART_ASSERT(p);
        // m_str = 'p'
        std::string m_str;
    };
    typedef std::vector< val_and_str> vals_array;

    // adds a value, and returns the size of existing values array
    int add_val( const std::string & val, const std::string & str) {
        m_aVals.push_back( val_and_str( val, str));
        return m_aVals.size();
    }

    const vals_array & get_vals_array() const {
        return m_aVals;
    }

    
    ////////////////////////////////////////////
    // level + extra param (level_msg)

    void set_level( int nLevel) {
        m_nLevel = nLevel;
    }

    int get_level() const {
        return m_nLevel;
    }

    void set_level_msg( const char * strMsg) {
        m_strLevelMsg = strMsg;
    }

    const std::string & get_level_msg() const {
        return m_strLevelMsg;
    }

private:
    context_vals_collection m_collContextVals;

    std::string m_strExpr;

    vals_array m_aVals;

    int m_nLevel;

    std::string m_strLevelMsg;
};



} // namespace smart_assert

} // namespace boost


#endif