// smart_assert_defs.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_DEFS_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_DEFS_HPP_INCLUDED

#include <sstream>

namespace boost { 

namespace smart_assert {


/* 
    default ASSERT levels
    (you can add your own, just make sure it does not
    mix with this)

    IMPORTANT
    Convention:

    The bigger the level is, the more critical is the error.
    For instance, a warning has a very low level number.
    The debug level is more critical than a warning, etc.

    As I said, you can add your own custom levels, just make sure
    you stick to this convension.

    Do it because the current code (and future code, for this matter) 
    assumes this of you.
*/
enum {

    // default behavior - just loggs this assert
    // (a message is shown to the user to the console)
    lvl_warn = 100,

    // default behavior - asks the user what to do:
    // Ignore/ Retry/ etc.
    lvl_debug = 200,

    // default behavior - throws a std::runtime_error
    lvl_error = 300,

    // default behavior - dumps all assert context to console, 
    // and aborts
    lvl_critical = 1000
};






} // namespace smart_assert

} // namespace boost


// note: the user can define SMART_ASSERT_CONTEXT prior to 
// #including any of smart_assert*.hpp files, to use its own!
#ifndef SMART_ASSERT_CONTEXT

    #ifdef __FUNCTION__
    // ... use __FUNCTION__ as well...
    #define SMART_ASSERT_CONTEXT context("file",__FILE__).context("line",__LINE__).context("function",__FUNCTION__)
    #else
    #define SMART_ASSERT_CONTEXT context("file",__FILE__).context("line",__LINE__)
    #endif


#endif // SMART_ASSERT_CONTEXT

#endif // BOOST_INNER_SMART_ASSERT_DEFS_HPP_INCLUDED
