// better_assert.cpp 
//

#pragma warning (disable : 4786)


// uncomment this, for the SMART_ASSERTs to behave just like
// in Release mode (that is, turning off SMART_ASSERTions)
//
// #define BOOST_SMART_ASSERT_MODE 0

#include <iostream>
#include <string>
#include "boost/smart_assert.hpp"
using namespace boost::smart_assert;


void test_first_second( int first, int second) {
    if ( first < second)
        SMART_ASSERT( first > second)(first)(second);
    else
        SMART_ASSERT( first < second)(first)(second);
}

void test_assert()
{
    int i, j, k;
    k = 1;
    i = 3;
    j = 2;
    SMART_ASSERT( (i < j) || (i == 3) || (j == 1) ) (i)(j);
    SMART_ASSERT( (i < j) || (i < 0) || (k == -1) ) (i)(j)(k);

    // shortcut, instead of SMART_ASSERT(p)(p);
    int *p = 0;
    SMART_ASSERT(p);

    test_first_second( 1, 2);
    test_first_second( 2, 1);
}


// should NEVER be called twice!
int function_once() {
    static int idx = 0;
    if ( ++idx > 1)
        std::cout << "ERROR: this function should NEVER be called twice" << std::endl;
    return 0;
}

void test_assert_use_v_() {
    int i, j;
    i = 3;
    j = 2;
    // the v_ macro should be used ONLY when the tested
    // function might take a while (the tested function is slow);
    // 
    // then, we would not want the slow function to be called twice, 
    // in case the assertion fails (see below).



    // use the v_ macro!
    //
    // this way, the function_once() will be called only once!
    //
    // if we were to use
    // 'SMART_ASSERT( function_once() || (i == j) )(function_once())(i)(j)';
    // function_once() would be called twice, since the assertion fails
    SMART_ASSERT( v_(function_once()) || (i == j) )(i)(j);
}

void test_assert_different_levels()
{
    // LEVEL warning
    int nUsersCount = 900;
    SMART_ASSERT( (nUsersCount * 10/9) < 1000) (nUsersCount)
        .warn( "Users Count almost reaching max. capacity (1000)!");


    // LEVEL debug
    int i = 3000;
    SMART_ASSERT( i < 100) (i).debug( "index too big!");

    // custom level;
    // this will also dump to console a "[SMART_ASSERT logical error]"
    // which is totally correct, since we have not set the handler
    // for level 500
    //
    //
    //
    // shortcut, instead of SMART_ASSERT(p)(p);
    int *p = 0;
    SMART_ASSERT(p).level( 500, "some custom level (500, that is)");


    // LEVEL error
    try {
        int i = 0, j = 0;
        SMART_ASSERT(i != j)(i)(j).error( "values should be different!");
#ifdef BOOST_SMART_ASSERT_DEBUG
        std::cout << "SHOULD NEVER ARRIVE HERE" << std::endl;
#endif
    }
    catch ( std::exception & exc) {
        std::cout << exc.what() << std::endl;
    }

    
    // LEVEL critical
    //
    // note: this, by default, would abort.
    // However, we change this, to just dump to console
    assert_settings().set_level_handler(
        lvl_critical, &print_details_to_console);
    std::string strUserName = "titi";
    SMART_ASSERT( strUserName == "admin")(strUserName)
        .critical( "User is not an administrator!");
    // set the default handler back
    assert_settings().set_level_handler(
        lvl_critical, &default_handle_critical);

}

void test_verify() {

    int i, j, k;
    k = 1;
    i = 3;
    j = 2;

    // usage - just like SMART_ASSERT !!!
    try {
        SMART_VERIFY( (i < j) || (i < 0) || (k == -1) ) (i)(j)(k);
        std::cout << "SHOULD NEVER ARRIVE HERE!!!" << std::endl;
    }
    catch ( std::exception & exc) {
        std::cout << exc.what() << std::endl;
    }

    // using the .msg function
    try {
        SMART_VERIFY( (i < j) && (i == 3) && (j == 1) ) (i)(j)
            .msg( "as expected, this failed");
        std::cout << "SHOULD NEVER ARRIVE HERE!!!" << std::endl;
    }
    catch ( std::exception & exc) {
        std::cout << exc.what() << std::endl;
    }

    // setting level to 'debug'
    int *p = 0;
    SMART_VERIFY(p).debug( "we'll ask the user what to do");

}


int main()
{
    //test_enforce();
    test_assert();
    test_assert_use_v_();
    test_assert_different_levels();
    test_verify();

    std::cout << "\nTest over. Press a non-space key." << std::endl;

    while ( isspace( std::cin.get() ));
    return 0;
}












// ignore testing Enforce for now...

#if 0
#include <io.h> // only on Windows platforms,

/* for Unix, instead of #include <io.h>

#include <unistd.h>
#define _read read
*/

struct Test
{
    Test( int i) : m_i( i)
    {}
    void print_i()
    { std::cout << "test=" << m_i << std::endl; }
private:
    int m_i;
};


struct HandlePredicate
{
    static bool Wrong(long handle)
    {
        return handle == -1;
    }
};
#define HANDLE_ENFORCE(exp) \
    boost::smart_assert::MakeEnforcer<HandlePredicate, boost::smart_assert::DefaultRaiser>((exp), "Expression '" #exp "' failed in '" \
    __FILE__ "', line: ", __LINE__).SMART_ENFORCE_A



void test_enforce()
{
    int i = 0;
    int *p = &i;

    int * p2 = ENFORCE( p)(p);
    *p2 = 1;

    try {
        int * p3 = ENFORCE( (int*)0);
    } catch( ...)
    {}

    Test t( 6);
    Test * pTest = &t;
    ENFORCE(pTest)(pTest)->print_i();

    // will of course faile
    int file = -1;
    void *buffer = NULL;
    int buflen = 40;
    try {
        const int available = 
            HANDLE_ENFORCE(_read(file, buffer, buflen))(file)(buffer)(buflen);
    }
    catch (...)
    {}

}

#endif
